#region Copyright Syncfusion Inc. 2001 - 2006
//
//  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.IO;

using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Styles;


namespace GridSample
{
	public class TreeCellModel : GridGenericControlCellModel
	{
		public TreeCellModel(GridModel grid)
			: base(grid)
		{
			TreeViewStyleProperties.Initialize(); 
			
		}
	

		protected TreeCellModel(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}


		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new TreeCellRenderer(control, this);
		}

		static Size treeViewControlSize = Size.Empty;

		public static Size TreeViewControlSize 
		{
			get
			{
				if (treeViewControlSize.IsEmpty)
				{
					// Determine default size of MonthCalendar when shown in a form
					TopLevelWindow f = new TopLevelWindow();
					f.Location = new Point(10000, 10000);					
					f.Size = new Size(500, 500);
					f.ShowWindowTopMost();
					TreeView drawTreeView = new TreeView();
					f.Controls.Add(drawTreeView);
					f.Visible = true;
					treeViewControlSize = drawTreeView.Size;
					f.Dispose();
				}
				return treeViewControlSize;
			}
		}



		protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
		{
			Size size = TreeViewControlSize;
			size.Width += 8;
			size.Height += 8;
			return size;
		}

		public static void InitializeCalendar(TreeView c, GridStyleInfo style)
		{
			TreeViewStyleProperties tree = new TreeViewStyleProperties(style);
			style.Control = tree.TreeView; 
		}

	}
	
	public class TreeCellRenderer: GridGenericControlCellRenderer
	{
		TreeView  drawTreeView = new TreeView();
		TreeView editTreeView = new TreeView();
		TreeCellModel cm;
		
		public TreeCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{

			cm = (TreeCellModel) cellModel;
			SupportsFocusControl = true;
			FixControlParent(drawTreeView );
			SetControl(editTreeView );
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				editTreeView.Dispose();
				editTreeView  = null;
				drawTreeView.Dispose();
				drawTreeView  = null;
				cm = null;
			}

			base.Dispose (disposing);
		}

		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			this.CurrentCell.IsModified = true; 

			base.OnInitialize (rowIndex, colIndex);
			TreeViewStyleProperties tree = new TreeViewStyleProperties(this.Grid.Model[RowIndex,ColIndex] );
			
		}

		protected override void OnActivated()
		{
			// Works around some interaction problem with PropertyGrid specific to this sample: force redrawing of
			// calendar after property grid was refreshed.
			Timer t = new Timer();
			t.Tick += new EventHandler(t_Tick);
			t.Interval = 200;
			t.Start();
			base.OnActivated ();

		}

		private void t_Tick(object sender, EventArgs e)
		{
			Timer t = (Timer) sender;
			t.Tick += new EventHandler(t_Tick);
			t.Dispose();
			
			editTreeView.Refresh();
		}
		
		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{
			clientRectangle.Size = GridUtil.Max(clientRectangle.Size, TreeCellModel.TreeViewControlSize);
			TreeCellModel.InitializeCalendar(drawTreeView , style);
			base.OnDraw (g, clientRectangle, rowIndex, colIndex, style);
		}

		
		protected override Rectangle OnLayout(int rowIndex, int colIndex, GridStyleInfo style, Rectangle innerBounds, Rectangle[] buttonsBounds)
		{
			Rectangle r = base.OnLayout (rowIndex, colIndex, style, innerBounds, buttonsBounds);

			r.Inflate(-1, -1);
			
			r = GridUtil.CenterInRect(r, TreeCellModel.TreeViewControlSize);
			r.Size = GridUtil.Max(r.Size, TreeCellModel.TreeViewControlSize);
			return r;
		}
	}
	/// Provides custom properties specific for PictureBox control. All properties
	/// support the style inheritance mechanism. You can change these propeties
	/// in the property grid.
	/// </summary>
	public class TreeViewStyleProperties : GridStyleInfoCustomProperties
	{
		static Type t = typeof(TreeViewStyleProperties);

		readonly static StyleInfoProperty TreeViewProperty = CreateStyleInfoProperty(t, "TreeView");
		
		static TreeViewStyleProperties defaultObject;

		static TreeViewStyleProperties()
		{
			// all properties must be initialized for the Default property
			defaultObject = new TreeViewStyleProperties(GridStyleInfo.Default);
			defaultObject.TreeView = null;
			
			
		}

		public static TreeViewStyleProperties Default
		{
			get
			{
				return defaultObject;
			}
		}

		public static void Initialize()
		{
		}

		public static explicit operator TreeViewStyleProperties(GridStyleInfo style)
		{
			return new TreeViewStyleProperties(style);
		}

		public TreeViewStyleProperties(GridStyleInfo style)
			: base(style)
		{
		}

		public TreeViewStyleProperties()
			: base()
		{
		}

		public TreeView  TreeView 
		{
			get
			{
				//TraceUtil.TraceCurrentMethodInfo();
				return (TreeView) style.GetValue(TreeViewProperty);
			}
			set
			{
				//TraceUtil.TraceCurrentMethodInfo(value);
				style.SetValue(TreeViewProperty, value);
			}
		}
		public void ResetSizeMode()
		{
			style.ResetValue(TreeViewProperty);
		}
		[EditorBrowsableAttribute(EditorBrowsableState.Never)]
		private bool ShouldSerializeTreeView()
		{
			return style.HasValue(TreeViewProperty);
		}
	}

}

